open Cmdliner
open Logarion
module C = Archive.Configuration
module Lpath = Logarion.Lpath

let conf () =
  let module Config = Confix.Config.Make (Confix.ConfixToml) in
  let archive_res =
    let open Confix.Config in
    Confix.Config.Path.with_file ".logarion/config.toml"
    &> Config.from_path
    |> Config.to_record C.of_config
  in
  match archive_res with
  | Ok config -> config
  | Error str -> prerr_endline str; exit 1

let init force =
  let rec create_dirs = function
    | [] -> ()
    | (dir,descr)::tl ->
       match Bos.OS.Dir.create (Fpath.v dir) with
       | Ok true -> print_endline ("Created " ^ descr ^ " directory in " ^ dir); create_dirs tl
       | Ok false -> print_endline ("Reinitialise existing " ^ descr ^ " directory in " ^ dir); create_dirs tl
       | Error (`Msg msg) -> prerr_endline @@ "Failed making " ^ descr ^ ". " ^ msg
  in
  let rec make_copies = function
    | [] -> ()
    | (source,dest)::tl ->
       let recursive = true in
       match Bos.OS.Cmd.run (Bos.Cmd.(v "cp" %% (on recursive @@ v "-r") % source % dest)) with
       | Ok () -> make_copies tl
       | Error (`Msg msg) -> prerr_endline msg
  in
  let dirs = [
      ".logarion", "Logarion";
      ".logarion/static", "static files";
      ".logarion/html-templates", "templates";
    ]
  in
  let copies = [
      "config.toml", ".logarion/";
      "share/static", ".logarion/";
      "share/html_templates", ".logarion/";
    ]
  in
  create_dirs dirs;
  make_copies copies

let init_term =
  let force =
    let doc = "Initialise repository even if directory is non empty" in
    Arg.(value & flag & info ["f"; "force"] ~doc)
  in
  Term.(const init $ force),
  Term.info
    "init" ~doc:"initialise a logarion repository in present directory"
    ~man:[ `S "DESCRIPTION"; `P "Create a repository in current directory" ]

let create_term =
  let title =
    Arg.(value & pos 0 string "" & info [] ~docv:"TITLE" ~doc:"Title for new article")
  in
  let f title =
    let repo = (conf ()).C.repository in
    let t = match title with "" -> "Draft" | _ -> title in
    let note = Note.({ (blank ()) with meta = { (Meta.blank ()) with Meta.title = t }}) in
    File.Lwt.with_note (File.store repo) note
    |> Lwt_main.run
    |> ignore
  in
  Term.(const f $ title),
  Term.info "create"
            ~doc:"create a new article"
            ~man:[ `S "DESCRIPTION";  `P "Create a new article, with title 'Draft' when none provided"]

let default_cmd =
  Term.(ret (const (`Help (`Pager, None)))),
  Term.info "logarion" ~version:"0.1.0" ~doc:"an article collection & publishing system"
            ~man:[ `S "BUGS"; `P "Submit bugs https://gitlab.com/orbifx/logarion/issues/new."; ]

let cmds = [ init_term; create_term ]

let () = match Term.eval_choice default_cmd cmds with
  | `Error _ -> exit 1 | _ -> exit 0
